* Confirmatory Factor Analysis (CFA) of gender-related questions in the World Values Survey
* Uses MPlus through runmplus module in Stata

* Last updated September 3, 2018

cd // Change to working director

clear 
set obs 0
gen country_year = ""
save cfa_by_country_year.dta, replace //creating blank data file to append results to

use wvs_selected_variables_all_respondents_2.dta, clear

levelsof(country_year), local(samples)
di `samples'
local samples = "AND2005 ARG2006 AUS2012  BFA2007 BGR2005 BHR2014 BLR2011 BRA2006 BRA2014 CAN2006 CHE2007 CHL2006 CHL2011 CHN2007 CHN2012  COL2012 CYP2006 CYP2011 DEU2006 DEU2013 DZA2013 ECU2013  EGY2013 ESP2007 ESP2011 EST2011 ETH2007 FIN2005 FRA2006 GBR2005 GEO2009 GHA2007 GHA2012  HKG2013 HUN2009 IDN2006 IND2006 IND2014  IRQ2012 JOR2014 JPN2005 JPN2010  KGZ2011 KOR2005 KOR2010 KWT2014 LBN2013 LBY2014 MAR2007  MDA2006 MEX2005 MEX2012 MLI2007 MYS2006 MYS2012 NGA2011 NLD2006 NLD2012 NOR2007 NZL2011 PAK2012  PER2012 PHL2012 POL2005 POL2012 PSE2013 ROU2005 ROU2012 RUS2006 RUS2011 RWA2012 SGP2012 SRB2005 SVN2005 SVN2011 SWE2006 SWE2011 THA2007  TTO2006 TTO2011 TUN2013 TUR2007 TUR2011 TWN2006 TWN2012 UKR2006 UKR2011 URY2006 URY2011 USA2011 UZB2011 YEM2014 ZAF2006 ZAF2013 ZMB2007 ZWE2012"
		
	//ISSUES with certain countries/variables:
			//1. SINGLE is missing for all country_years after 2009  !Deleting it from GENDER 
			//   a. Not all categories used in  AUS2005 EGY2008 JOR2007 USA2006 !Deleting countries from analyses for now
			//   ?:  Did they use a different version of the question with 2 response options??
			//2. EQRIGHTS also has missing data issues
			//   a. No variation in  RWA2007 QAT2010                  !Deleting countries from analyses for now
			//	 b. Not asked in COL2005 HKG2005  ITA2005             !Deleting countries from analyses
			//3. BEATWIFE
			//   a. Not asked in EGY2008 HKG2005 IRQ2006 PER2006      !Deleting countries from analyses
			//   b. Not all categories used in all countries(ok if we treat as continuous)
			//4. JOBS
			//   a. Not asked in COL2005 							  !Deleting countries from analyses
			//   b. Not all categories used in IRQ2006                !Deleting countries from analyses
			//5. CEO
			//   a. Not asked in HKG2005 IRQ2006					  !Deleting countries from analyses
			//6. UNIV
			//   a. Not asked in HKG2005 IRQ2006					  !Deleting countries from analyses
			//7. POL_LEAD...all looks ok!
			//8. WOMENMVT
			//   a. Not asked in IRQ2006							  !Deleting country from analyses
			//   b. Not all categories used in VNM2006                !Deleting country from analyses
			//9. HOUSEWIF
			//   a. Not asked in IRN2007							  !Deleting countries from analyses
			
		//other country-years causing problems (not sure why, but it won't run with these countries included)
		//		KAZ2011 MAR2011  THA2013
		
		
foreach sample of local samples {
	preserve
	keep if country_year=="`sample'" & female==1 //women only for this part
		*"`sample'"
	log using countrylog, replace
	runmplus eqrights beatwife jobs ceo univ  pol_lead womenmvt housewif, ///
		categorical(eqrights  jobs ceo univ  pol_lead womenmvt housewif) ///  treating beatwife as continuous
		model(gender by eqrights beatwife jobs ceo univ  pol_lead womenmvt housewif;) ///
	*runmplus_fits   CFI TLI RMSEA //saving fit statistics
		*cant get CHI2 or its DF to work here, so finding it via text search below
	*mat list e(fits) //Get fit statistics as matrix...couldn't figure out how to merge this back in, so I just got the data via text search like above
	log close _all
	
	import delimited countrylog.smcl, clear
	
	local N = _N
	gen temp = _n if trim(v1)=="Chi-Square Test of Model Fit"
	egen tempx = max(temp)
	local chi2line = tempx in 1
	local chi2 = v1 in `=`chi2line'+1'
	di "`chi2'"
	local df = v1 in `=`chi2line'+2'
	local pvalue = v1 in `=`chi2line'+3'
	local rmsea = v1 in `=`chi2line'+9' 
	local cfi = v1 in `=`chi2line'+13' 
	local tli = v1 in `=`chi2line'+14' 
	
	*clear
	*set obs 1
	gen temp1 = trim("`chi2'")
	replace temp1 = substr(temp1,10,.)
	gen chi2 = real(substr(temp1,1,strlen(temp1)-1))
	li
	gen temp2 = trim("`df'")
	egen temp3 = sieve(temp2), keep(numeric) //part of egenmore package
	gen df = real(temp3)
	li
	gen temp4 = trim("`pvalue'")
	li
	gen pvalue = real(substr(temp4,10,.))
	li
	gen temp5 = trim("`rmsea'")
	li
	gen rmsea = real(substr(temp5,10,.))
	li
	gen temp6 = trim("`cfi'")
	li
	gen cfi = real(substr(temp6,10,.))
	li
	gen temp7 = trim("`tli'")
	li
	gen tli = real(substr(temp7,10,.))
	li
	drop temp*
	
	gen N = `N' // add in sample size
	gen country_year = "`sample'" // add country-year
	gen bic = chi2-(df*ln(N)) //add bic
	
	
	*Now adding in factor loadings and their p-values
		*Pulling in data from the output
			gen temp_fl = _n if trim(v1)=="Estimate       S.E.  Est./S.E.    P-Value"
			egen temp_flx = max(temp_fl)
			local fl_eqrights_line = temp_flx in 1
			local fl_eqrights = v1 in `=`fl_eqrights_line'+2'
			local fl_beatwife = v1 in `=`fl_eqrights_line'+3'
			local fl_jobs = v1 in `=`fl_eqrights_line'+4'
			local fl_ceo = v1 in `=`fl_eqrights_line'+5'	
			local fl_univ = v1 in `=`fl_eqrights_line'+6'
			local fl_pol_lead = v1 in `=`fl_eqrights_line'+7'
			local fl_womenmvt = v1 in `=`fl_eqrights_line'+8'
			local fl_housewif = v1 in `=`fl_eqrights_line'+9'
			
			di "`fl_eqrights'"
			di "`fl_housewif'" 
	
		*Now cleaning it up to generate the factor laoding and p-value for each
			*Can't just loop through all of the variables because they are different lengths
			
			*Variables with 8 characters
				foreach v in  beatwife pol_lead womenmvt housewif {
					gen temp_fl_`v' = subinstr("`fl_`v''"," ","",.) //removing spaces within line
					
					*Identifying the location of the second . 
					gen splitspot_`v' = strpos(subinstr(temp_fl_`v', ".", ",", 1), ".")
					local splitspot = splitspot_`v' - 10  //(-2 to include the . and value before it, and minus 8 for var name)
					
					gen fl_`v' = substr(temp_fl_`v', 9, `splitspot') //numbers before second . are the factor loading
					gen pv_`v' = real(substr(temp_fl_`v', -5, 5)) //p value is the last 5 (has to be <1)
				}
				
				*list temp_fl_beatwife fl_beatwife pv_beatwife in 1
				*list temp_fl_housewif fl_housewif pv_housewif in 1
					//works both with -s and +s!
				
			*Variables with 4 characters
				foreach v in jobs univ {
					gen temp_fl_`v'= subinstr("`fl_`v''"," ","",.) //removing spaces within line
					*Identifying the location of the second . 
					gen splitspot_`v' = strpos(subinstr(temp_fl_`v', ".", ",", 1), ".")
					local splitspot = splitspot_`v' - 6  //(-2 to include the . and value before it, and minus 4 for var name)
					
					gen fl_`v' = substr(temp_fl_`v', 5, `splitspot') //numbers before second . are the factor loading
					gen pv_`v' = real(substr(temp_fl_`v', -5, 5)) //p value is the last 5 (has to be <1)
				}
				
				*list temp_fl_jobs fl_jobs pv_jobs in 1
				*list temp_fl_univ fl_univ pv_univ in 1
				
				
			*And ceo with 3 characters
				gen temp_fl_ceo = subinstr("`fl_ceo'"," ", "", .)
				*Identifying the location of the second . 
					gen splitspot_ceo = strpos(subinstr(temp_fl_ceo, ".", ",", 1), ".")
					local splitspot = splitspot_ceo - 5 //(-2 to include the . and value before it, and minus 3 for var name)
					
					gen fl_ceo = substr(temp_fl_ceo, 4, `splitspot') //numbers before second . are the factor loading
					gen pv_ceo = real(substr(temp_fl_ceo, -5, 5)) //p value is the last 5 (has to be <1)
	
				*list temp_fl_ceo fl_ceo pv_ceo in 1
	
	
	*Note that I am not generating a fl or pvalue for eqrights because it is the scaling factor
	
	
	*Keeping only necessary variables
	keep chi2 df pvalue rmsea cfi tli N country_year bic fl_beatwife pv_beatwife fl_pol_lead pv_pol_lead fl_womenmvt pv_womenmvt fl_housewif pv_housewif fl_jobs pv_jobs fl_univ pv_univ fl_ceo pv_ceo
	*Keeping only one case per country
	keep if _n==1
	
	append using cfa_by_country_year.dta
	save cfa_by_country_year.dta, replace
	
	restore
	}

	
	use cfa_by_country_year.dta, clear
	sort bic
	browse
	
	
	*Generating flag for number of nonsignificant factor loadings
		foreach v in pv_beatwife pv_pol_lead pv_womenmvt pv_housewif pv_jobs pv_univ pv_ceo {
			gen `v'_flag = 1 if `v' >0.0499999 & `v' !=.
		}
		egen numpvns = rowtotal(pv_beatwife_flag pv_pol_lead_flag pv_womenmvt_flag pv_housewif_flag pv_jobs_flag pv_univ_flag pv_ceo_flag)
		*list numpvns pv_beatwife* pv_pol_lead* pv_womenmvt* pv_housewif* pv_jobs* pv_univ* pv_ceo* in 1/10
		
	save cfa_by_country_year.dta, replace
	
*Merging with main dataset	
	merge 1:m country_year using wvs_selected_variables.dta
	keep country_year pvalue bic rmsea cfi tli HDI GIIi fl_* pv_beatwife pv_pol_lead pv_womenmvt pv_housewif pv_jobs pv_univ pv_ceo numpvns
	sort country_year
	
	bysort country_year: gen seq=_n
	bysort country_year: keep if _n==_N //keeping just one observation per country
	drop seq

	drop HDI
	save cfa_by_country_year_women.dta, replace
